# --
# Kernel/System/Auth/HTTPBasicAuth.pm - provides the $ENV authentication
# Copyright (C) 2001-2009 OTRS AG, http://otrs.org/
# --
# $Id: HTTPBasicAuth.pm,v 1.16 2009/09/22 15:19:27 mb Exp $
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --
# Note:
#
# If you use this module, you should use as fallback the following
# config settings:
#
# If use isn't login through apache ($ENV{REMOTE_USER} or $ENV{HTTP_REMOTE_USER})
# $Self->{LoginURL} = 'http://host.example.com/not-authorised-for-otrs.html';
#
# $Self->{LogoutURL} = 'http://host.example.com/thanks-for-using-otrs.html';
# --

package Kernel::System::Auth::OpMon;

use strict;
use warnings;
use DBI;
use Data::Dumper;

use vars qw($VERSION);
$VERSION = qw($Revision: 0.1 $) [1];

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    my $Self = {};
    bless( $Self, $Type );

    # check needed objects
    for (qw(LogObject ParamObject ConfigObject DBObject)) {
        $Self->{$_} = $Param{$_} || die "No $_!";
    }

	# OpMon Config File
	$Self->{dbConfigFile} = $Self->{ConfigObject}->Get( 'OpMon::DBConfigFile' );
	if (!($Self->{dbConfigFile})) {
		$Self->{LogObject}->Log(
			Priority => 'error',
			Message  => "Need OpMon::DBConfigFile in Kernel/Config.pm",
		);
		return;
    }

    # Debug 0=off 1=on
    $Self->{Debug} = 0;
    $Self->{Count} = $Param{Count} || '';

    return $Self;
}

sub GetOption {
    my ( $Self, %Param ) = @_;

    # check needed stuff
    if ( !$Param{What} ) {
        $Self->{LogObject}->Log( Priority => 'error', Message => "Need What!" );
        return;
    }

    # module options
    my %Option = ( PreAuth => 1, );

    # return option
    return $Option{ $Param{What} };
}

sub Auth {
    my ( $Self, %Param ) = @_;

	my $Request = $Self->{ParamObject}->GetParam( Param => 'RequestedURL' );
	my @pairs = split(/&/, $Request);
	foreach my $pair (@pairs) {
		my ($name, $value) = split(/=/,$pair);
		$Param{$name} = $value;
	}
	
    # get params
	my $User = $Param{User} || '';
	my $Pw = $Param{Pw} || '';
	my $SessionID = $Param{SessionID} || '';

    # return on no user
    if ( !$User ) {
        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message =>
                "User: No User!.",
        );
        return;
    }

	# read cookies
	my %cookies = fetch CGI::Cookie;
	if (defined($cookies{'OPMONSSID'})) {
		my $MySessionID = $cookies{'OPMONSSID'}->value;
		if ($MySessionID eq $SessionID) {
			return $User;
		}
	}

	# parse config file
	my ($dbuser,$dbpass,$dbhost);
	open( FILE, $Self->{dbConfigFile} );
	while(<FILE>) {
		$dbuser = $1 if (m/DBUSER\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
		$dbpass = $1 if (m/DBPASS\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
		$dbhost = $1 if (m/DBHOST\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
	}
	close(FILE);

	# validate password
	my $dbh = DBI->connect( "dbi:mysql:dbname=opcfg;host=$dbhost", "$dbuser", "$dbpass" );
	if (!defined($dbh)) {
		$Self->{LogObject}->Log( Priority => 'error', Message => "Can't connect to database!" );
		return;
	}

	my $sth = $dbh->prepare("SELECT contact_id FROM nagios_contacts WHERE contact_name = '$User' AND password = md5('$Pw')");
	if (!defined($sth)) {
		$Self->{LogObject}->Log( Priority => 'error', Message => "Cannot prepare statement: $DBI::errstr!" );
		return;
	}

	$sth->execute;
	my @Data = $sth->fetchrow_array();
	if (!(@Data)) {
		$Self->{LogObject}->Log( Priority => 'error', Message => "Wrong password for user $User!" );
		return;
	}
	$sth->finish;

    # return login
    return $User;
}
1;
